errormessage = (
    "h5py does not seem to be installed. You will not be able to use HDF5 files through "
    "TransparentPath.\nYou can change that by running 'pip install transparentpath[h5py]'."
)
hdf5_ok = False


class TPImportError(ImportError):
    def __init__(self, message: str = ""):
        self.message = f"Error in TransparentPath: {message}"
        super().__init__(self.message)


class MyHDFFile:
    def __init__(self):
        raise TPImportError(errormessage)


class MyHDFStore:
    def __init__(self):
        raise TPImportError(
            "pandas does not seem to be installed. You will not be able to use pandas objects through "
            "TransparentPath.\nYou can change that by running 'pip install transparentpath[pandas]'."
        )


try:
    import warnings
    import h5py
    import tempfile
    from typing import Union, Any
    from pathlib import Path
    from ..gcsutils.transparentpath import TransparentPath, TPValueError, TPFileNotFoundError
    from .pandas import MyHDFStore
    import sys
    import importlib.util

    hdf5_ok = True

    if importlib.util.find_spec("tables") is None:
        raise TPImportError("Need the 'tables' package")

    class MyHDFFile(h5py.File):
        """Class to override h5py.File to handle files on GCS.

        This allows to do :
        >>> from transparentpath import TransparentPath  # doctest: +SKIP
        >>> import numpy as np  # doctest: +SKIP
        >>> TransparentPath.set_global_fs("gcs", bucket="bucket_name")  # doctest: +SKIP
        >>> path = TransparentPath("chien.hdf5"  # doctest: +SKIP
        >>>
        >>> with path.write() as ifile:  # doctest: +SKIP
        >>>     ifile["data"] = np.array([1, 2])  # doctest: +SKIP
        """

        def __init__(self, *args, remote: Union[TransparentPath, None] = None, **kwargs):
            """Overload of h5py.File.__init__, to accept a 'remote' argument

            Parameters
            ----------
            args: tuple
                First argument is the local path to read. It can be a TransparentPath or a
                tempfile._TemporaryFileWrapper
            remote: Union[TransparentPath, None]
                Path to the file on GCS. Only relevant if file is opened in a 'with' statement in write mode. It
                will be used to put the modified local temporary HDF5 back to GCS. If specified, args[0] is expected to
                be a tempfile._TemporaryFileWrapper (Default value = None).
            kwargs: dict
            """
            self.remote_file = remote
            self.local_path = args[0]
            # noinspection PyUnresolvedReferences,PyProtectedMember
            if isinstance(self.local_path, tempfile._TemporaryFileWrapper):
                # noinspection PyUnresolvedReferences
                h5py.File.__init__(self, args[0].name, *args[1:], **kwargs)
            else:
                h5py.File.__init__(self, *args, **kwargs)

        def __exit__(self, *args):
            """Overload of the h5py.File.__exit__ method to push any modified local temporary HDF5 back to GCS after
            a 'with' statement."""
            h5py.File.__exit__(self, *args)
            if self.remote_file is not None:
                # noinspection PyUnresolvedReferences
                TransparentPath(self.local_path.name, fs="local").put(self.remote_file)
                # noinspection PyUnresolvedReferences
                self.local_path.close()

    def read(self: TransparentPath, use_pandas: bool = False, **kwargs,) -> Union[h5py.File, MyHDFStore]:
        """Reads a HDF5 file. Must have been created by h5py.File or pd.HDFStore (specify use_pandas=True if so)

        Since h5py.File/pd.HDFStore does not support GCS, first copy it in a tmp file.


        Parameters
        ----------
        self: TransparentPath

        use_pandas: bool
            To use HDFStore instead of h5py.File (Default value = False)

        kwargs
            The kwargs to pass to h5py.File/pd.HDFStore method, or to dask.dataframe.read_hdf()


        Returns
        -------
        Union[h5py.File, MyHDFStore]
            Opened h5py.File/pd.HDFStore

        """

        mode = "r"
        if "mode" in kwargs:
            mode = kwargs["mode"]
            del kwargs["mode"]
        if "r" not in mode:
            raise TPValueError("If using read_hdf5, mode must contain 'r'")

        class_to_use = h5py.File
        if use_pandas:
            class_to_use = MyHDFStore

        if not self.is_file():
            raise TPFileNotFoundError(f"Could not find file {self}")

        if self.fs_kind == "local":
            # Do not check kwargs since HDFStore and h5py both accepct kwargs anyway
            data = class_to_use(self.path, mode=mode, **kwargs)
        else:
            f = tempfile.NamedTemporaryFile(delete=False, suffix=".hdf5")
            f.close()  # deletes the tmp file, but we can still use its name
            self.get(f.name)
            # Do not check kwargs since HDFStore and h5py both accepct kwargs anyway
            data = class_to_use(f.name, mode=mode, **kwargs)
            Path(f.name).unlink()
        return data

    def write(
        self: TransparentPath, data: Any = None, set_name: str = None, use_pandas: bool = False, **kwargs,
    ) -> Union[None, h5py.File, MyHDFStore]:
        """

        Parameters
        ----------
        self: TransparentPath
        data: Any
            The data to store. Can be None, in that case an opened file is returned (Default value = None)
        set_name: str
            The name of the dataset (Default value = None)
        use_pandas: bool
            To use pd.HDFStore object instead of h5py.File (Default = False)
        **kwargs

        Returns
        -------
        Union[None, pd.HDFStore, h5py.File]

        """

        if self.suffix != ".hdf5" and self.suffix != "h5":
            warnings.warn(f"path {self} does not have '.h(df)5' as suffix while using to_hdf5. The path will be "
                          f"changed to a path with '.hdf5' as suffix")
            self.change_suffix(".hdf5")

        mode = "w"
        if "mode" in kwargs:
            mode = kwargs["mode"]
            del kwargs["mode"]

        if self.when_checked["used"] and not self.nocheck:
            self._check_multiplicity()

        # If no data is specified, an HDF5 file is returned, opened in write mode, or any other specified mode.
        if data is None:

            class_to_use = MyHDFFile
            if use_pandas:
                class_to_use = MyHDFStore

            if self.fs_kind == "local":
                return class_to_use(self.path, mode=mode, **kwargs)
            else:
                f = tempfile.NamedTemporaryFile(delete=True, suffix=".hdf5")
                return class_to_use(f, remote=self.path, mode=mode, **kwargs)
        else:

            if isinstance(data, dict):
                sets = data
            else:
                if set_name is None:
                    set_name = "data"
                sets = {set_name: data}

            class_to_use = h5py.File
            if use_pandas:
                class_to_use = MyHDFStore

            if self.fs_kind == "local":
                thefile = class_to_use(self.path, mode=mode, **kwargs)
                for aset in sets:
                    thefile[aset] = sets[aset]
                thefile.close()
            else:
                with tempfile.NamedTemporaryFile(delete=True, suffix=".hdf5") as f:
                    thefile = class_to_use(f.name, mode=mode, **kwargs)
                    for aset in sets:
                        thefile[aset] = sets[aset]
                    thefile.close()
                    TransparentPath(
                        path=f.name,
                        fs="local",
                        notupdatecache=self.notupdatecache,
                        nocheck=self.nocheck,
                        when_checked=self.when_checked,
                        when_updated=self.when_updated,
                        update_expire=self.update_expire,
                        check_expire=self.check_expire,
                    ).put(self.path)

    try:
        # noinspection PyUnresolvedReferences
        from .pandas import MyHDFStore as Hs

        MyHDFStore = Hs
    except ImportError:
        pass

except ImportError as e:
    raise TPImportError(str(e))



import zipfile
import pytest
from ..functions import init, skip_gcs

from transparentpath import TransparentPath


# noinspection PyUnusedLocal
@pytest.mark.parametrize("fs_kind", ["local", "gcs"])
def test_zipfile(clean, fs_kind):
    if skip_gcs[fs_kind]:
        print("skipped")
        return
    init(fs_kind)
    if fs_kind == "local":
        data_path = TransparentPath("tests/data/chien.zip")
    else:
        local_path = TransparentPath("tests/data/chien.zip", fs_kind="local")
        data_path = TransparentPath("chien.zip")
        local_path.put(data_path)

    zf = zipfile.ZipFile(data_path)
    text1 = zf.open("text.txt")
    text2 = zf.open("text 2.txt")
    assert text1.read() == b'chien\n'
    assert text2.read() == b'chat\n'



import pytest
import sys
import importlib.util
from importlib import reload
from transparentpath import TransparentPath
from .functions import init, skip_gcs, get_reqs
from pathlib import Path

requirements = get_reqs(Path(__file__).stem.split("test_")[1])

reqs_ok = True
for req in requirements:
    if importlib.util.find_spec(req) is None:
        reqs_ok = False
        break


dic = {"animals": {"chien": 4, "bird": 2}, "plants": {"drosera": "miam", "celeri": "beurk"}}


# noinspection PyUnusedLocal
@pytest.mark.parametrize("fs_kind", ["local", "gcs"])
def test_read_json(clean, fs_kind):
    reload(sys.modules["transparentpath"])
    if skip_gcs[fs_kind]:
        print("skipped")
        return
    init(fs_kind)

    if fs_kind == "local":
        data_path = TransparentPath("tests/data/chien.json")
    else:
        local_path = TransparentPath("tests/data/chien.json", fs_kind="local")
        data_path = TransparentPath("chien.json")
        local_path.put(data_path)

    if reqs_ok:
        res = data_path.read()
        assert res == dic
    else:
        with pytest.raises(ImportError):
            data_path.read()


# noinspection PyUnusedLocal
@pytest.mark.parametrize(
    "fs_kind", ["local", "gcs"]
)
def test_write_dict(clean, fs_kind):
    if skip_gcs[fs_kind]:
        print("skipped")
        return
    init(fs_kind)

    path = get_path(fs_kind)
    data = dic
    if reqs_ok:
        path.write(data=data)
        assert path.is_file()
        res = path.read()
        assert res == data
    else:
        with pytest.raises(ImportError):
            path.write(data=data)


# noinspection PyUnusedLocal
@pytest.mark.parametrize(
    "fs_kind", ["local", "gcs"]
)
def test_write_numpy(clean, fs_kind):
    if skip_gcs[fs_kind]:
        print("skipped")
        return
    init(fs_kind)

    path = get_path(fs_kind)
    if reqs_ok:
        # noinspection PyUnresolvedReferences
        import numpy as np
        data = np.array([[1, 2, 3], [4, 5, 6]])
        path.write(data=data)
        assert path.is_file()
        res = path.read()
        # noinspection PyUnresolvedReferences
        assert (res == data).all()


# noinspection PyUnusedLocal
@pytest.mark.parametrize(
    "fs_kind", ["local", "gcs"]
)
def test_write_pandas(clean, fs_kind):
    if skip_gcs[fs_kind]:
        print("skipped")
        return
    init(fs_kind)

    path = get_path(fs_kind)
    if importlib.util.find_spec("pandas") is not None and reqs_ok:
        # noinspection PyUnresolvedReferences
        import pandas as pd
        # noinspection PyUnresolvedReferences
        import numpy as np
        data = pd.DataFrame(
            data=np.array([[1, 2, 3], [4, 5, 6]]),
            columns=["A", "B", "C"],
            index=[10, 30],
        )
        path.write(data=data)
        assert path.is_file()
        res = path.read()
        # noinspection PyUnresolvedReferences
        assert (np.array(res["data"]) == data.values).all()
        assert res["columns"] == data.columns.tolist()
        assert res["index"] == data.index.tolist()


def get_path(fs_kind):
    reload(sys.modules["transparentpath"])
    if skip_gcs[fs_kind]:
        print("skipped")
        return "skipped"
    init(fs_kind)

    pcsv = TransparentPath("chien.json")
    pcsv.rm(absent="ignore", ignore_kind=True)
    assert not pcsv.is_file()
    return pcsv



import pytest
import importlib.util
from .functions import get_reqs, get_path
from pathlib import Path

requirements = get_reqs(Path(__file__).stem.split("test_")[1])

reqs_ok = True
for req in requirements:
    if "[" in req:
        main = req.split("[")[0]
        if importlib.util.find_spec(main) is None:
            reqs_ok = False
            break
        req = req.replace(main, "").replace("[", "").replace("]", "")
        req = req.split(",")
        for r in req:
            if r == "distributed":
                if importlib.util.find_spec(r) is None:
                    reqs_ok = False
                    break
            elif r == "dataframe":
                continue
            else:
                raise AssertionError(f"I do not know what to do with {r} in requirement {req}")

    else:
        if importlib.util.find_spec(req) is None:
            reqs_ok = False
            break


# noinspection PyUnusedLocal,PyShadowingNames
@pytest.mark.parametrize(
    "fs_kind, suffix, kwargs",
    [
        ("local", ".csv", {"index_col": 0}), ("gcs", ".csv", {"index_col": 0}),
        ("local", ".parquet", {}), ("gcs", ".parquet", {}),
        ("local", ".xlsx", {"index_col": 0}), ("gcs", ".xlsx", {"index_col": 0}),
        ("local", ".hdf5", {"set_names": "data"}), ("gcs", ".hdf5", {"set_names": "data"}),
    ]
)
def test(clean, fs_kind, suffix, kwargs):
    if reqs_ok is False:
        pfile = get_path(fs_kind, suffix)
        with pytest.raises(ImportError):
            pfile.read(use_dask=True)
    else:
        # noinspection PyUnresolvedReferences
        import pandas as pd
        import dask.dataframe as dd
        df_dask = dd.from_pandas(
            pd.DataFrame(
                columns=["foo", "bar"],
                index=["a", "b"],
                data=[[1, 2], [3, 4]]
            ),
            npartitions=1
        )
        # noinspection PyTypeChecker
        pfile = get_path(fs_kind, suffix)
        if pfile == "skipped":
            return
        pfile.write(df_dask)
        assert pfile.is_file()
        pd.testing.assert_frame_equal(df_dask.head(), pfile.read(use_dask=True, **kwargs).head())



import importlib.util
from ..functions import init, skip_gcs, get_reqs
from transparentpath import TransparentPath
import os


requirements = get_reqs("vanilla")

for req in requirements:
    if importlib.util.find_spec(req) is None:
        raise ImportError(f"TransparentPath needs {req} package")


def test_multiproject_1(clean):
    if skip_gcs["gcs"]:
        print("skipped")
        return
    init("gcs")
    TransparentPath.set_global_fs("gcs", token=os.environ["GOOGLE_APPLICATION_CREDENTIALS_2"])
    p1 = TransparentPath("code_tests_sand")
    p2 = TransparentPath("code_tests_dev")
    p3 = TransparentPath("coucou")
    assert "gcs" in p1.fs_kind
    assert "gcs" in p2.fs_kind
    assert p1.fs_kind != p2.fs_kind
    assert p3.fs_kind == "local"


def test_multiproject_2(clean):
    if skip_gcs["gcs"]:
        print("skipped")
        return
    init("gcs")
    TransparentPath("code_test_sand")
    TransparentPath.set_global_fs("gcs", bucket="code_tests_dev", token=os.environ["GOOGLE_APPLICATION_CREDENTIALS_2"])
    p1 = TransparentPath("code_tests_sand")
    p2 = TransparentPath("code_tests_dev")
    p3 = TransparentPath("coucou")
    assert "gcs" in p1.fs_kind
    assert "gcs" in p2.fs_kind
    assert "gcs" in p3.fs_kind
    assert p1.fs_kind != p2.fs_kind
    assert p3.fs_kind == p2.fs_kind



errormessage = (
    "parquet for TransparentPath does not seem to be installed. You will not be able to use parquet files "
    "through TransparentPath.\nYou can change that by running 'pip install transparentpath[parquet]'."
)

parquet_ok = False


class TPImportError(ImportError):
    def __init__(self, message: str = ""):
        self.message = f"Error in TransparentPath: {message}"
        super().__init__(self.message)


try:
    import pandas as pd
    import tempfile
    import warnings
    import sys
    from typing import Union, List, Tuple
    import importlib.util
    from ..gcsutils.transparentpath import TransparentPath, check_kwargs, TPFileExistsError, TPFileNotFoundError

    if importlib.util.find_spec("pyarrow") is None:
        raise TPImportError("Need the 'pyarrow' package")

    parquet_ok = True

    def get_index_and_date_from_kwargs(**kwargs: dict) -> Tuple[int, bool, dict]:
        index_col = kwargs.get("index_col", None)
        parse_dates = kwargs.get("parse_dates", None)
        if index_col is not None:
            del kwargs["index_col"]
        if parse_dates is not None:
            del kwargs["parse_dates"]
        # noinspection PyTypeChecker
        return index_col, parse_dates, kwargs

    def apply_index_and_date(index_col: int, parse_dates: bool, df: pd.DataFrame) -> pd.DataFrame:
        if index_col is not None:
            df = df.set_index(df.columns[index_col])
            df.index = df.index.rename(None)
        if parse_dates is not None:
            # noinspection PyTypeChecker
            df.index = pd.to_datetime(df.index)
        return df

    def read(self, **kwargs) -> Union[pd.DataFrame, pd.Series]:

        if not self.is_file():
            raise TPFileNotFoundError(f"Could not find file {self}")

        index_col, parse_dates, kwargs = get_index_and_date_from_kwargs(**kwargs)

        check_kwargs(pd.read_parquet, kwargs)
        if self.fs_kind == "local":
            return apply_index_and_date(
                index_col, parse_dates, pd.read_parquet(self.__fspath__(), engine="pyarrow", **kwargs)
            )

        else:
            return apply_index_and_date(
                index_col, parse_dates, pd.read_parquet(self.open("rb"), engine="pyarrow", **kwargs)
            )

    def write(
        self,
        data: Union[pd.DataFrame, pd.Series],
        overwrite: bool = True,
        present: str = "ignore",
        columns_to_string: bool = True,
        to_dataframe: bool = True,
        compression: str = None,
        **kwargs,
    ) -> None:
        """
        Warning : if data is a Dask dataframe, the output will be written in a directory. For convenience, the directory
        if self.with_suffix(""). Reading is transparent and one can specify a path with .parquet suffix.
        """

        if self.suffix != ".parquet":
            warnings.warn(f"path {self} does not have '.parquet' as suffix while using to_parquet. The path will be "
                          f"changed to a path with '.parquet' as suffix")
            self.change_suffix(".parquet")

        if compression is not None and compression != "snappy":
            warnings.warn("TransparentPath can not write parquet files with a compression that is not snappy. You "
                          f"specified '{compression}', it will be replaced by 'snappy'.")

        if not overwrite and self.is_file() and present != "ignore":
            raise TPFileExistsError()

        if to_dataframe and isinstance(data, pd.Series):
            name = data.name
            data = pd.DataFrame(data=data)
            if name is not None:
                data.columns = [name]

        if columns_to_string and not isinstance(data.columns[0], str):
            # noinspection PyUnresolvedReferences
            data.columns = data.columns.astype(str)

        # noinspection PyTypeChecker
        check_kwargs(data.to_parquet, kwargs)
        data.to_parquet(self.open("wb"), engine="pyarrow", compression="snappy", **kwargs)


except ImportError as e:
    raise TPImportError(str(e))



import pytest
from pathlib import Path


acceptable = {
    "pandas": ["numpy"],
    "hdf5": ["numpy"],
    "parquet": ["numpy"],
    "excel": ["numpy"],
    "dask": ["pandas", "numpy"]
}


@pytest.fixture
def reqs(pytestconfig):
    name = pytestconfig.getoption("name")
    requirements = {}
    all_reqs = []

    for afile in Path("").glob("*requirements.txt"):
        if afile.stem == "requirements":
            opt = "vanilla"
        else:
            opt = afile.stem.split("-requirements")[0]

        requirements[opt] = []
        for s in afile.read_text().splitlines():
            s = s.split("==")[0] if "==" in s else s
            s = s.split("<=")[0] if "<=" in s else s
            s = s.split(">=")[0] if ">=" in s else s
            s = s.split("<")[0] if "<" in s else s
            s = s.split(">")[0] if ">=" in s else s
            s = s.split("[")[0] if "[" in s else s
            requirements[opt].append(s)
        all_reqs = list(set(all_reqs) | set(requirements[opt]))

    if name == "all":
        return all_reqs, [], []

    this_one = requirements[name]
    others = []

    for opt in requirements:
        if opt == name or opt == "vanilla":
            continue
        else:
            others += [r for r in requirements[opt] if r not in this_one and r not in others]
    return this_one, others, acceptable[name] if name in acceptable else []


def pytest_addoption(parser):
    parser.addoption("--name", action="store", default=None)



import pytest
import importlib.util
from transparentpath import TransparentPath
from transparentpath.gcsutils.transparentpath import TPMultipleExistenceError
from pathlib import Path
from ..functions import init, skip_gcs, get_reqs

requirements = get_reqs(Path(__file__).stem.split("test_")[1])

cc = "chien/chat"

for req in requirements:
    if importlib.util.find_spec(req) is None:
        raise ImportError(f"TransparentPath needs {req} package")


# noinspection PyUnusedLocal
@pytest.mark.parametrize(
    "what, test_against, expected",
    [
        (TransparentPath, TransparentPath, True),
        (TransparentPath, Path, False),
        (TransparentPath, str, True),
        (Path, TransparentPath, False),
        (Path, str, False),
        (Path, Path, True),
        (str, str, True),
        (str, Path, False),
        (str, TransparentPath, False),
    ],
)
def test_isinstance(clean, what, test_against, expected):
    assert isinstance(what(), test_against) is expected


# noinspection PyUnusedLocal
def test_collapse_dots(clean):
    assert TransparentPath("chien/chat/../../../cheval") == TransparentPath("../cheval")


# noinspection PyUnusedLocal
@pytest.mark.parametrize("fs_kind, excep", [("local", FileExistsError), ("gcs", TPMultipleExistenceError)])
def test_multipleexistenceerror(clean, fs_kind, excep):
    if skip_gcs[fs_kind]:
        print("skipped")
        return
    init(fs_kind)

    # noinspection PyTypeChecker
    with pytest.raises(excep):
        p1 = TransparentPath("chien")
        p2 = TransparentPath("chien") / "chat"
        p2.touch()
        if excep == FileExistsError:
            p1.touch()
        else:
            p1.fs.touch(p1.__fspath__())
        TransparentPath("chien").read()


# noinspection PyUnusedLocal
@pytest.mark.parametrize("fs_kind", ["local", "gcs"])
def test_equal(clean, fs_kind):
    if skip_gcs[fs_kind]:
        print("skipped")
        return
    init(fs_kind)
    p1 = TransparentPath("chien")
    p2 = TransparentPath("chien")
    assert p1 == p2


# noinspection PyUnusedLocal
@pytest.mark.parametrize("fs_kind", ["local", "gcs"])
def test_lt(clean, fs_kind):
    if skip_gcs[fs_kind]:
        print("skipped")
        return
    init(fs_kind)
    p1 = TransparentPath("chien") / "chat"
    p2 = TransparentPath("chien")
    assert p2 < p1


# noinspection PyUnusedLocal
@pytest.mark.parametrize("fs_kind", ["local", "gcs"])
def test_gt(clean, fs_kind):
    if skip_gcs[fs_kind]:
        print("skipped")
        return
    init(fs_kind)
    p1 = TransparentPath("chien") / "chat"
    p2 = TransparentPath("chien")
    assert p1 > p2


# noinspection PyUnusedLocal
@pytest.mark.parametrize("fs_kind", ["local", "gcs"])
def test_le(clean, fs_kind):
    if skip_gcs[fs_kind]:
        print("skipped")
        return
    init(fs_kind)
    p1 = TransparentPath("chien") / "chat"
    p2 = TransparentPath("chien")
    p3 = TransparentPath("chien")
    assert p2 <= p1
    assert p3 <= p2


# noinspection PyUnusedLocal
@pytest.mark.parametrize("fs_kind", ["local", "gcs"])
def test_gt(clean, fs_kind):
    if skip_gcs[fs_kind]:
        print("skipped")
        return
    init(fs_kind)
    p1 = TransparentPath("chien") / "chat"
    p2 = TransparentPath("chien")
    p3 = TransparentPath("chien")
    assert p1 >= p2
    assert p3 >= p2


# noinspection PyUnusedLocal
@pytest.mark.parametrize("fs_kind", ["local", "gcs"])
def test_contains(clean, fs_kind):
    if skip_gcs[fs_kind]:
        print("skipped")
        return
    init(fs_kind)
    p1 = TransparentPath("chien") / "chat"
    assert "chi" in p1


# noinspection PyUnusedLocal
@pytest.mark.parametrize("fs_kind", ["local", "gcs"])
def test_add(clean, fs_kind):
    if skip_gcs[fs_kind]:
        print("skipped")
        return
    init(fs_kind)
    p1 = TransparentPath("chien")
    assert (p1 + "chat") == TransparentPath(cc)
    assert (p1 + "/chat") == TransparentPath(cc)


# noinspection PyUnusedLocal
@pytest.mark.parametrize("fs_kind", ["local", "gcs"])
def test_truediv(clean, fs_kind):
    if skip_gcs[fs_kind]:
        print("skipped")
        return
    init(fs_kind)
    p1 = TransparentPath("chien")
    assert (p1 / "chat") == TransparentPath(cc)
    assert (p1 / "/chat") == TransparentPath(cc)


# noinspection PyUnusedLocal
@pytest.mark.parametrize("fs_kind", ["local", "gcs"])
def test_itruediv(clean, fs_kind):
    if skip_gcs[fs_kind]:
        print("skipped")
        return
    init(fs_kind)
    p1 = TransparentPath("chien")
    p1 /= "chat"
    assert p1 == TransparentPath(cc)
    p1 = TransparentPath("chien")
    p1 /= "/chat"
    assert p1 == TransparentPath(cc)


# noinspection PyUnusedLocal
@pytest.mark.parametrize(
    "fs_kind, path1, path2, expected",
    [
        ("local", "chien/chat", "chien", True),
        ("gcs", "chien/chat", "chien", True),
        ("local", "chien/chat", "chien", True),
        ("gcs", "chien/chat", "chien", True),
        ("local", "chien", "chien", False),
        ("gcs", "chien", "chien", False),
        ("local", "chien", "chien", False),
        ("gcs", "chien", "chien", False),
        ("local", "chien", "chat", False),
        ("gcs", "chien", "chat", False),
        ("local", "chien", "chat", False),
    ],
)
def test_isdir(clean, fs_kind, path1, path2, expected):
    if skip_gcs[fs_kind]:
        print("skipped")
        return
    init(fs_kind)
    p1 = TransparentPath(path1)
    p1.touch()
    p2 = TransparentPath(path2)
    assert p2.is_dir() == expected


# noinspection PyUnusedLocal
@pytest.mark.parametrize(
    "fs_kind, path1, path2, expected",
    [
        ("local", "chien/chat", "chien/chat", True),
        ("gcs", "chien/chat", "chien/chat", True),
        ("local", "chien/chat", "chien", False),
        ("gcs", "chien/chat", "chien", False),
        ("local", "chien", "chien", True),
        ("gcs", "chien", "chien", True),
        ("local", "chien", "chat", False),
        ("gcs", "chien", "chat", False),
    ],
)
def test_isfile(clean, fs_kind, path1, path2, expected):
    if skip_gcs[fs_kind]:
        print("skipped")
        return
    init(fs_kind)
    p1 = TransparentPath(path1)
    p1.touch()
    p2 = TransparentPath(path2)
    assert p2.is_file() == expected


# noinspection PyUnusedLocal
@pytest.mark.parametrize(
    "fs_kind, path1, path2, kwargs, expected",
    [
        ("local", "chien", "chien", {"absent": "raise", "ignore_kind": False, "recursive": False}, None),
        ("gcs", "chien", "chien", {"absent": "raise", "ignore_kind": False, "recursive": False}, None),
        ("local", "chien", "chien", {"absent": "raise", "ignore_kind": False, "recursive": True}, NotADirectoryError),
        ("gcs", "chien", "chien", {"absent": "raise", "ignore_kind": False, "recursive": True}, NotADirectoryError),
        ("local", "ch/chat", "ch", {"absent": "raise", "ignore_kind": False, "recursive": False}, IsADirectoryError),
        ("gcs", "ch/chat", "ch", {"absent": "raise", "ignore_kind": False, "recursive": False}, IsADirectoryError),
        ("local", "ch/chat", "ch", {"absent": "raise", "ignore_kind": True, "recursive": False}, None),
        ("gcs", "ch/chat", "ch", {"absent": "raise", "ignore_kind": True, "recursive": False}, None),
        ("local", "ch/chat", "ch", {"absent": "raise", "ignore_kind": False, "recursive": True}, None),
        ("gcs", "ch/chat", "ch", {"absent": "raise", "ignore_kind": False, "recursive": True}, None),
        ("local", "", "ch", {"absent": "raise", "ignore_kind": False, "recursive": False}, FileNotFoundError),
        ("gcs", "", "ch", {"absent": "raise", "ignore_kind": False, "recursive": False}, FileNotFoundError),
        ("local", "", "ch", {"absent": "raise", "ignore_kind": False, "recursive": True}, NotADirectoryError),
        ("gcs", "", "ch", {"absent": "raise", "ignore_kind": False, "recursive": True}, NotADirectoryError),
        ("local", "", "ch", {"absent": "ignore", "ignore_kind": False, "recursive": True}, None),
        ("gcs", "", "ch", {"absent": "ignore", "ignore_kind": False, "recursive": True}, None),
    ],
)
def test_rm(clean, fs_kind, path1, path2, kwargs, expected):
    if skip_gcs[fs_kind]:
        print("skipped")
        return
    init(fs_kind)
    p1 = TransparentPath(path1)
    p2 = TransparentPath(path2)
    if path1 != "":
        p1.touch()
    if expected is not None:
        with pytest.raises(expected):
            p2.rm(**kwargs)
    else:
        p2.rm(**kwargs)
        assert not p2.exists()


# noinspection PyUnusedLocal
@pytest.mark.parametrize(
    "fs_kind, pattern, expected",
    [
        ("local", "chien/*", ["chat", "cheval"]),
        ("local", "chien/**", ["chat", "cheval", "cheval/chouette"]),
        ("gcs", "chien/*", ["chat", "cheval"]),
        ("gcs", "chien/**", ["chat", "cheval", "cheval/chouette"]),
    ],
)
def test_glob(clean, fs_kind, pattern, expected):
    if skip_gcs[fs_kind]:
        print("skipped")
        return
    init(fs_kind)

    dic = {"chien": "dir", "chien/chat": "file", "chien/cheval": "dir", "chien/cheval/chouette": "file"}
    root = TransparentPath()
    for word in dic:
        p = root / word
        p.rm(absent="ignore", ignore_kind=True)
    for word in dic:
        p = root / word
        if dic[word] == "file":
            p.touch()
        else:
            p.mkdir()
    print(list(TransparentPath("chien").ls()))
    content = [str(p).split("chien/")[1] for p in TransparentPath().glob(pattern)]
    assert content == expected
    for word in dic:
        p = root / word
        p.rm(absent="ignore", ignore_kind=True)


# noinspection PyUnusedLocal
@pytest.mark.parametrize(
    "fs_kind, suffix, expected",
    [("local", ".txt", ".txt"), ("local", "txt", ".txt"), ("gcs", ".txt", ".txt"), ("gcs", "txt", ".txt")],
)
def test_with_suffix(clean, fs_kind, suffix, expected):
    if skip_gcs[fs_kind]:
        print("skipped", fs_kind, suffix, expected)
        return
    init(fs_kind)

    p = TransparentPath("chien").with_suffix(suffix)
    assert p.suffix == expected


# noinspection PyUnusedLocal
@pytest.mark.parametrize("fs_kind", ["local", "gcs"])
def test_ls(clean, fs_kind):
    if skip_gcs[fs_kind]:
        print("skipped")
        return
    init(fs_kind)

    root = TransparentPath()
    (root / "chien").mkdir()
    (root / "chien" / "chat").touch()
    (root / "chien" / "cheval").mkdir()
    (root / "chien" / "cheval" / "chouette").touch()
    res = [str(p).split("chien/")[1] for p in (root / "chien").ls()]
    res.sort()
    expected = ["chat", "cheval"]
    expected.sort()
    assert res == expected


# noinspection PyUnusedLocal
@pytest.mark.parametrize("fs_kind", ["local", "gcs"])
def test_cd(clean, fs_kind):
    if skip_gcs[fs_kind]:
        print("skipped")
        return
    init(fs_kind)

    root = TransparentPath()
    (root / "chien").mkdir()
    (root / "chien" / "chat").touch()
    (root / "chien" / "cheval").mkdir()
    (root / "chien" / "cheval" / "chouette").touch()
    root.cd("chien")
    assert root == TransparentPath("chien")
    root.cd("..")
    assert root == TransparentPath()


# noinspection PyUnusedLocal
@pytest.mark.parametrize(
    "fs_kind, path", [("local", "chien/chat"), ("local", "chien"), ("gcs", "chien/chat"), ("gcs", "chien")]
)
def test_touch(clean, fs_kind, path):
    if skip_gcs[fs_kind]:
        print("skipped")
        return
    init(fs_kind)

    p = TransparentPath(path)
    if p.exists():
        p.rm(ignore_kind=True)
    p.touch()
    assert p.is_file()


# noinspection PyUnusedLocal
@pytest.mark.parametrize(
    "fs_kind, path, expected",
    [("local", "chien", True), ("local", "chien/chat", True), ("gcs", "chien", False), ("gcs", "chien/chat", False)],
)
def test_mkdir(clean, fs_kind, path, expected):
    if skip_gcs[fs_kind]:
        print("skipped")
        return
    init(fs_kind)

    p = TransparentPath(path)
    p.mkdir()
    assert p.is_dir() is expected


# noinspection PyUnusedLocal
@pytest.mark.parametrize("fs_kind, to_append", [("local", "chat"), ("gcs", "chat")])
def test_append(clean, fs_kind, to_append):
    if skip_gcs[fs_kind]:
        print("skipped")
        return
    init(fs_kind)

    assert str(TransparentPath("chien").append(to_append)) == str(TransparentPath(f"chien{to_append}"))


# noinspection PyUnusedLocal
@pytest.mark.parametrize("fs_kind", ["local", "gcs"])
def test_walk(clean, fs_kind):
    if skip_gcs[fs_kind]:
        print("skipped")
        return
    init(fs_kind)

    dic = {"chien": "dir", "chien/chat": "file", "chien/cheval": "dir", "chien/cheval/chouette": "file"}
    expected = [
        (
            TransparentPath("chien"),
            [TransparentPath("chien") / "cheval"],
            [TransparentPath("chien") / "chat"],
        ),
        (TransparentPath("chien") / "cheval", [], [TransparentPath("chien") / "cheval" / "chouette"]),
    ]
    root = TransparentPath()
    for word in dic:
        p = root / word
        p.rm(absent="ignore", ignore_kind=True)
    for word in dic:
        p = root / word
        if dic[word] == "file":
            p.touch()
        else:
            p.mkdir()

    assert list((root / "chien").walk()) == expected
    for word in dic:
        p = root / word
        p.rm(absent="ignore", ignore_kind=True)


# noinspection PyUnusedLocal
@pytest.mark.parametrize("fs_kind, ", ["local", "gcs"])
def test_exists(clean, fs_kind):
    if skip_gcs[fs_kind]:
        print("skipped")
        return
    init(fs_kind)

    p = TransparentPath("chien")
    p.touch()
    assert p.exist()
    assert p.exists()


# noinspection PyUnusedLocal
def test_buckets(clean):
    if skip_gcs["gcs"]:
        print("skipped")
        return
    init("gcs")

    assert "code_tests_sand/" in TransparentPath().buckets



errormessage = (
    "pandas does not seem to be installed. You will not be able to use pandas objects through "
    "TransparentPath.\nYou can change that by running 'pip install transparentpath[pandas]'."
)


class TPImportError(ImportError):
    def __init__(self, message: str = ""):
        self.message = f"Error in TransparentPath: {message}"
        super().__init__(self.message)


try:
    import pandas as pd
    import tempfile
    import warnings
    from typing import Union, List
    from ..gcsutils.transparentpath import TransparentPath, check_kwargs, TPFileExistsError, TPFileNotFoundError

    class MyHDFStore(pd.HDFStore):
        """Same as MyHDFFile but for pd.HDFStore objects"""

        def __init__(self, *args, remote: Union[TransparentPath, None] = None, **kwargs):
            self.remote_file = remote
            self.local_path = args[0]
            # noinspection PyUnresolvedReferences,PyProtectedMember
            if isinstance(self.local_path, tempfile._TemporaryFileWrapper):
                pd.HDFStore.__init__(self, args[0].name, *args[1:], **kwargs)
            else:
                pd.HDFStore.__init__(self, *args, **kwargs)

        def __exit__(self, exc_type, exc_value, traceback):
            pd.HDFStore.__exit__(self, exc_type, exc_value, traceback)
            if self.remote_file is not None:
                TransparentPath(self.local_path.name, fs="local").put(self.remote_file)
                self.local_path.close()

    def read(self, **kwargs) -> pd.DataFrame:

        if not self.is_file():
            raise TPFileNotFoundError(f"Could not find file {self}")

        # noinspection PyTypeChecker,PyUnresolvedReferences
        try:

            check_kwargs(pd.read_csv, kwargs)
            with self.fs.open(self.__fspath__()) as f:
                return pd.read_csv(f, **kwargs)

        except pd.errors.ParserError:
            # noinspection PyUnresolvedReferences
            raise pd.errors.ParserError(
                "Could not read data. Most likely, the file is encrypted."
                " Ask your cloud manager to remove encryption on it."
            )

    def write(
        self, data: Union[pd.DataFrame, pd.Series], overwrite: bool = True, present: str = "ignore", **kwargs,
    ):

        if self.suffix != ".csv":
            warnings.warn(f"path {self} does not have '.csv' as suffix while using to_csv. The path will be "
                          f"changed to a path with '.csv' as suffix")
            self.change_suffix(".csv")

        if not overwrite and self.is_file() and present != "ignore":
            raise TPFileExistsError()

        check_kwargs(data.to_csv, kwargs)
        with self.fs.open(self.__fspath__(), "w") as f:
            data.to_csv(f, **kwargs)


except ImportError as e:
    raise TPImportError(str(e))



import zipfile
import tempfile
from pathlib import Path
from ..gcsutils.transparentpath import TransparentPath

zipfileclass = zipfile.ZipFile


class TpZipFile(zipfileclass):
    """
    Overload of ZipFile class to handle files on GCS
    """

    def __init__(self, path, *args, **kwargs):
        if type(path) == TransparentPath and path.fs_kind != "local":
            if path.when_checked["used"] and not path.nocheck:
                # noinspection PyProtectedMember
                path._check_multiplicity()
            f = tempfile.NamedTemporaryFile(delete=False, suffix=".zip")
            f.close()  # deletes tmp file, but we can still use its name
            path.get(f.name)
            path = Path(f.name)  # Path is pathlib, not TransparentPath
            super().__init__(path, *args, **kwargs)
            path.unlink()
        else:
            super().__init__(path, *args, **kwargs)


zipfile.ZipFile = TpZipFile
